@@ -121,6 +121,12 @@ Atom net_wm_window_type_utility;
 Atom net_workarea;
 Atom utf8_string;
 
+/* Systray similation for older KDE apps */
+Atom kde_net_wm_system_tray_window_for;
+Atom net_system_tray_manager;
+Atom net_system_tray_selection;
+Atom net_system_tray_opcode;
+
 static gboolean
 check_type_and_format (Display * dpy, Window w, Atom a, int expected_format,
     Atom expected_type, int n_items, int format, Atom type)
@@ -210,7 +216,7 @@ getMotifHints (Display * dpy, Window w)
     {
         if (items_read >= MWM_HINTS_ELEMENTS)
         {    
-            result = g_new(PropMwmHints, 1);
+            result = g_new0(PropMwmHints, 1);
             memcpy (result, data, sizeof (PropMwmHints));
         }
         XFree (data);
@@ -291,6 +297,21 @@ initGnomeHints (Display * dpy)
     win_workspace = XInternAtom (dpy, "_WIN_WORKSPACE", FALSE);
 }
 
+void
+initSystrayHints (Display * dpy, int nscreen)
+{
+    gchar selection[32];
+
+    TRACE ("entering initSystrayHints");
+
+    g_snprintf (selection, sizeof (selection), "_NET_SYSTEM_TRAY_S%d", nscreen);
+    net_system_tray_manager   = XInternAtom (dpy, "MANAGER", FALSE);
+    net_system_tray_opcode    = XInternAtom (dpy, "_NET_SYSTEM_TRAY_OPCODE", FALSE);
+    net_system_tray_selection = XInternAtom (dpy, selection, FALSE);
+    kde_net_wm_system_tray_window_for = 
+        XInternAtom (dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
+}
+
 gboolean
 getHint (Display * dpy, Window w, Atom a, long *value)
 {
@@ -325,12 +346,12 @@ setHint (Display * dpy, Window w, Atom a, long value)
 }
 
 void
-getGnomeDesktopMargins (Display * dpy, int screen, CARD32 * m)
+getGnomeDesktopMargins (Display * dpy, int screen, int * m)
 {
     Atom real_type;
     int real_format;
     unsigned long items_read, items_left;
-    CARD32 *data = NULL;
+    unsigned long *data = NULL;
 
     TRACE ("entering getGnomeDesktopMargins");
 
@@ -339,10 +360,10 @@ getGnomeDesktopMargins (Display * dpy, int screen, CARD32 * m)
                 &real_type, &real_format, &items_read, &items_left,
                 (unsigned char **) &data) == Success) && (items_read >= 4))
     {
-        m[0] = data[0];
-        m[1] = data[1];
-        m[2] = data[2];
-        m[3] = data[3];
+        m[0] = (int) data[0];
+        m[1] = (int) data[1];
+        m[2] = (int) data[2];
+        m[3] = (int) data[3];
         XFree (data);
     }
     else
@@ -583,34 +604,29 @@ get_cardinal_list (Display * dpy, Window w, Atom xatom,
 }
 
 void
-set_net_workarea (Display * dpy, int screen, int nb_workspaces, CARD32 * m)
+set_net_workarea (Display * dpy, int screen, int nb_workspaces, int * m)
 {
-    CARD32 *data, *ptr;
+    unsigned long *data, *ptr;
     int i, j;
 
     TRACE ("entering set_net_workarea");
     j = (nb_workspaces ? nb_workspaces : 1);
-    data = (CARD32 *) malloc (sizeof (CARD32) * j * 4);
-    if (!data)
-    {
-        gdk_beep ();
-        return;
-    }
+    data = (unsigned long *) g_new0 (unsigned long, j * 4);
     ptr = data;
     for (i = 0; i < j; i++)
     {
-        *ptr++ = m[MARGIN_LEFT];
-        *ptr++ = m[MARGIN_TOP];
-        *ptr++ =
-            MyDisplayFullWidth (dpy,
-            screen) - (m[MARGIN_LEFT] + m[MARGIN_RIGHT]);
-        *ptr++ =
-            MyDisplayFullHeight (dpy,
-            screen) - (m[MARGIN_TOP] + m[MARGIN_BOTTOM]);
+        *ptr++ = (unsigned long) m[MARGIN_LEFT];
+        *ptr++ = (unsigned long) m[MARGIN_TOP];
+        *ptr++ = (unsigned long) 
+            (MyDisplayFullWidth (dpy, screen) - 
+                (m[MARGIN_LEFT] + m[MARGIN_RIGHT]));
+        *ptr++ = (unsigned long) 
+            (MyDisplayFullHeight (dpy, screen) - 
+                (m[MARGIN_TOP] + m[MARGIN_BOTTOM]));
     }
     XChangeProperty (dpy, RootWindow (dpy, screen), net_workarea, XA_CARDINAL,
         32, PropModeReplace, (unsigned char *) data, j * 4);
-    free (data);
+    g_free (data);
 }
 
 void
@@ -715,7 +731,11 @@ get_utf8_string (Display * dpy, Window w, Atom xatom, char **str_p)
         return FALSE;
     }
 
-    utf8_string_remove_controls((gchar *) str, -1, NULL);
+    if (str)
+    {
+        utf8_string_remove_controls((gchar *) str, -1, NULL);
+    }
+    
     *str_p = str;
 
     return TRUE;
@@ -757,6 +777,10 @@ get_text_property (Display * dpy, Window w, Atom a)
     if (XGetTextProperty (dpy, w, &text, a))
     {
         retval = text_property_to_utf8 (dpy, &text);
+        if (retval)
+        {
+            utf8_string_remove_controls((gchar *) retval, -1, NULL);
+        }
         if ((text.value) && (text.nitems > 0))
         {
             XFree (text.value);
@@ -771,7 +795,6 @@ get_text_property (Display * dpy, Window w, Atom a)
     return retval;
 }
 
-
 void
 getWindowName (Display * dpy, Window w, char **name)
 {
@@ -801,6 +824,69 @@ getWindowName (Display * dpy, Window w, char **name)
     }
 }
 
+gboolean
+checkKdeSystrayWindow(Display * dpy, Window window)
+{
+    Atom actual_type;
+    int actual_format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    Window trayIconForWindow;
+
+    TRACE ("entering GetWindowRole");
+    g_return_if_fail (window != None);
+    
+    XGetWindowProperty(dpy, window, kde_net_wm_system_tray_window_for, 0L, 
+            sizeof(Window), FALSE, XA_WINDOW, &actual_type, &actual_format, 
+            &nitems, &bytes_after, (unsigned char **)&trayIconForWindow);
+
+    if ((actual_format == None) || 
+        (actual_type != XA_WINDOW) || 
+        (trayIconForWindow == None))
+    {
+        return FALSE;
+    }
+    return TRUE;
+}
+
+void
+sendSystrayReqDock(Display * dpy, Window window, Window systray)
+{
+    XClientMessageEvent xev;
+
+    TRACE ("entering sendSystrayReqDock");
+    g_return_if_fail (window != None);
+    g_return_if_fail (systray != None);
+
+    xev.type = ClientMessage;
+    xev.window = systray;
+    xev.message_type = net_system_tray_opcode;
+    xev.format = 32;
+    xev.data.l[0] = CurrentTime;
+    xev.data.l[1] = 0; /* SYSTEM_TRAY_REQUEST_DOCK */
+    xev.data.l[2] = window;
+    xev.data.l[3] = 0; /* Nada */
+    xev.data.l[4] = 0; /* Niet */
+
+    XSendEvent (dpy, systray, FALSE, NoEventMask, (XEvent *) & xev);
+}
+
+Window
+getSystrayWindow (Display * dpy)
+{
+    Window systray_win = None;
+
+    TRACE ("entering getSystrayWindow");
+
+    systray_win = XGetSelectionOwner (dpy, net_system_tray_selection);
+    if (systray_win)
+    {
+        XSelectInput (dpy, systray_win, StructureNotifyMask);
+    }
+    TRACE ("New systray window:  0x%lx", systray_win);
+    return systray_win;
+}
+
 gboolean
 getWindowRole (Display * dpy, Window window, char **role)
 {
